##########################################################################
#
# StudioFactory
#
# The desktop Audio/Video studio.
# Copyright (C) 2002-2004  Peter Wendrich (pwsoft@syntiac.com)
# Homepage: http://www.syntiac.com/studiofactory.html
#
##########################################################################
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#
##########################################################################
#
# Project Browser window
#
##########################################################################

my @projectStructDef=(
  'bool selected;',
  'bool modifiedFlag;',
  'bool unfoldedProjectView;',
  'char *projectName;',
  'char *projectPath;',
  'DspObject *dsp;',
  'SynFactoryPatchPtr patches;',
  'TrackPtr tracks;',
  'SamplePtr samples;',
  'int maxPatches;',
  'int maxTracks;',
  'int maxSamples;',
  'int patchCount;',
);
#    'int trackCount;',
genEnumStruct("struct", "Project", \@projectStructDef);

rv('const char * const newProjectName="new project";');
rv('ProjectPtr projects;');
rv('int maxProjects;');
rv('int currentPlayingProject=-1;');
rv('int currentProject=-1;');
rv('int currentPatch=-1;');
rv('int projectBrowserPos;');

$rs .= <<EOF;
static void selectProjectBrowserItem(int project, SynFactoryPatchPtr patch, int object, int track, bool ctrlPressed) {
  int i,j,k;
  int p=0;

//  projectBrowserPos=-1;
  currentPatch=-1;
  for(i=0;i<maxProjects;i++) {
    if (project==i) {
      currentProject=i;
    }
    if (project==i && patch==NULL && track==-1) {
      projects[i].selected = true;
      projectBrowserPos = p;
      if (projects[i].maxPatches>0) {
        currentPatch=0;
      }
    } else {
      if (!ctrlPressed) {
        projects[i].selected=false;         
      }
    }
    p++;

    // Force selected item visible
    if (project==i && (patch!=NULL || track>=0)) {
      projects[i].unfoldedProjectView = true;
    }

    // Patches
    for(j=0;j<projects[i].maxPatches;j++) {
      if (project==i && patch==projects[i].patches+j) {
        currentPatch=j;
      }
      if (project==i && patch==projects[i].patches+j && object<0) {
        projects[i].patches[j].selected=true;
        if (projects[i].unfoldedProjectView) {
          projectBrowserPos = p;
        }
      } else {
        if (!ctrlPressed) {
          projects[i].patches[j].selected=false;
        }
      }
      if (projects[i].unfoldedProjectView) {
        p++;
      }
      for(k=0;k<projects[i].patches[j].maxObjects;k++) {
        if (project==i && patch==projects[i].patches+j && object==k) {
          projects[i].patches[j].objects[k].selected=true;
        } else {
          if (!ctrlPressed) {
            projects[i].patches[j].objects[k].selected=false;
          }
        }
      }
    }

    // Tracks
    for(j=0;j<projects[i].maxTracks;j++) {
      if (project==i && track==j) {
        projects[i].tracks[j].selected=true;
        if (projects[i].unfoldedProjectView) {
          projectBrowserPos = p;
        }
      } else {
        if (!ctrlPressed) {
          projects[i].tracks[j].selected=false;
        }
      }
      if (projects[i].unfoldedProjectView) {
        p++;
      }
    }
  }

//  projects[project].tracks[track].selected=true;
  projectBrowserWindowRefresh=true;
  mainWindowRefresh=true;
  patchConfigWindowRefresh=true;
}

static int createProject(const char *projectName) {
  int n=-1;

  logprintf("createProject\\n");

  n=maxProjects;
  maxProjects++;
  lockMutex(audioMutex);
  lockMutex(midiMutex);
  projects=(ProjectPtr)realloc(projects, maxProjects*sizeof(Project));
  unlockMutex(midiMutex);
  unlockMutex(audioMutex);
  memset(&(projects[n]), 0, sizeof(Project));
  projects[n].dsp=(DspObject*)calloc(maxDspObjects, sizeof(DspObject));
  if (projectName) {
    projects[n].projectName=strdup(projectName);
  }
  currentPatch=-1;
  selectProjectBrowserItem(n, NULL, -1, -1, false);
  transportWindowRefresh=true;

  logprintf("createProject returns %d\\n", n);
  return n;
}

static void setProjectModified(int project, bool aModifiedFlag) {
  if (project>=0 && project<maxProjects) {
    projects[project].modifiedFlag=aModifiedFlag;
    projectBrowserWindowRefresh=true;
  }  
}

static void setProjectPath(int project, const char *aPath) {
  if (project>=0 && project<maxProjects) {
    if (projects[project].projectPath) {
      free(projects[project].projectPath);
      projects[project].projectPath=NULL;
      projectBrowserWindowRefresh=true;
    }
    if (aPath) {
      projects[project].projectPath=strdup(aPath);
      projectBrowserWindowRefresh=true;
    }
  }
}
EOF

  $rc .= <<EOF;
static void deleteProject(int project) {
  int i;

  if (currentPlayingProject == project) {
    setPlayMode(PLAYMODE_STOP);
  } 

  for(i=0;i<projects[project].maxPatches;i++) {
    deleteSynFactoryObjects(project, &(projects[project].patches[i]), true);
  }

  lockMutex(audioMutex);
  lockMutex(midiMutex);
  free(projects[project].dsp);
  if (projects[project].patches) {
    free(projects[project].patches);
  }
  if (projects[project].projectName) {
    free(projects[project].projectName);
  }
  maxProjects--;
  for(i=project;i<maxProjects;i++) {
    projects[i]=projects[i+1];
  }
  projects=(ProjectPtr)realloc(projects, maxProjects*sizeof(Project));
  if (currentPlayingProject >= project) {
    currentPlayingProject--;
  }
  if (currentProject>=maxProjects) {
    currentProject=maxProjects-1;
  }
  if (currentProject>=0) {
    currentPatch=projects[currentProject].maxPatches-1;
  } else {
    currentPatch=-1;
  }
  projectBrowserWindowRefresh=true;
  patchConfigWindowRefresh=true;
  mainWindowRefresh=true;
  unlockMutex(midiMutex);
  unlockMutex(audioMutex);
}

static void deleteSelected(void) {
  int i=0;

  while(i<maxProjects) {
    if (projects[i].selected) {
      deleteProject(i);
    } else {
      int j=0;
      while (j<projects[i].maxPatches) {
        deleteSynFactoryObjects(i, &(projects[i].patches[j]), projects[i].patches[j].selected);
        if (projects[i].patches[j].selected) {
          for(int k=j;k<projects[i].maxPatches-1;k++) {
            projects[i].patches[k]=projects[i].patches[k+1];
          }
          projects[i].maxPatches--;
          projects[i].patches=(SynFactoryPatchPtr)realloc(projects[i].patches, projects[i].maxPatches*sizeof(SynFactoryPatch));
          if (currentProject==i && currentPatch>=projects[i].maxPatches) {
            currentPatch=projects[i].maxPatches-1;
          }

          projectBrowserWindowRefresh=true;
          mainWindowRefresh=true;
          patchConfigWindowRefresh=true;
        } else {
          j++;
        }
      }
      i++;
    }
  }
}

static void projectBrowserAddHitArea(ContextPtr aContext, int y) {
  guiAddArea(aContext, aContext->guiClientRect.left, y, aContext->guiClientRect.right, y+14);
}

static void drawExpandBox(ContextPtr aContext, int x, int y, bool expanded) {
  guiSelectFillColor(aContext, -1);
  guiDrawRect(aContext, x, y, x+14, y+12);
  MoveToEx(aContext->hdc, x+4, y+6, NULL);
  LineTo(aContext->hdc, x+11, y+6);
  if (!expanded) {
    MoveToEx(aContext->hdc, x+7, y+3, NULL);
    LineTo(aContext->hdc, x+7, y+10);            
  }
}

static int projectBrowserBackground(bool cursor, bool selected) {
  if (cursor) {
    if (selected) {
      return 0x00AAAA;
    } else {
      return 0xAAAAAA;
    }
  } else {
    if (selected) {
      return 0x00FFFF;
    } else {
      return 0xFFFFFF;
    }
  }
}

static void projectBrowserHandler(ContextPtr aContext) {
  static bool recalcCursor=false;
  static int maxy=0;
  static int maxysize=0;

  switch (aContext->guiEvent) {
  case GUIEVENT_REFRESH: {
      int textColor;
      int color;
      int y=-guiGetVScrollPos(aContext->currentWindow);
	  int ycursor=0;
      int i;
      int j;

      guiClearAreaList(aContext);

      guiSelectFillColor(aContext, 0xFFFFFF);
      guiSelectPen1Color(aContext, -1);
      guiDrawRect(aContext, aContext->guiClientRect.left, aContext->guiClientRect.top, aContext->guiClientRect.right, aContext->guiClientRect.bottom);

      guiSelectPotFont(aContext);
      guiTextTransparent(aContext, true);
      guiSelectPen1Color(aContext, 0);

      for(i=0;i<maxProjects;i++) {
        if (currentProject==i) {
          textColor=0;
        } else {
          textColor=0x888888;
        }

        guiSelectPen1Color(aContext, textColor);
        drawExpandBox(aContext, 2, y, projects[i].unfoldedProjectView);

        color=projectBrowserBackground(projectBrowserPos==guiGetAreaListLen(aContext), projects[i].selected);
		if (projectBrowserPos==guiGetAreaListLen(aContext)) ycursor=y;

        guiSelectPen1Color(aContext, -1);
        guiSelectFillColor(aContext, color);
        guiDrawRect(aContext, 18, y, aContext->guiClientRect.right, y+14);
        expandString(projects[i].projectName, (projects[i].modifiedFlag)?" *":NULL, " (", projects[i].projectPath, ")");
        guiSelectPen1Color(aContext, textColor);
        TextOut(aContext->hdc, 20, y, expandedString, strlen(expandedString));
        projectBrowserAddHitArea(aContext, y);
        y+=14;

//          guiSelectFillColor(aContext, 15);
        if (projects[i].unfoldedProjectView) {
          if (projects[i].maxPatches>0) {
            guiSelectFillColor(aContext, 0xFFFFFF);
            TextOut(aContext->hdc, 40, y, "Patches", 7);
            y+=14;
            for(j=0;j<projects[i].maxPatches;j++) {
              int k;

              color=projectBrowserBackground(projectBrowserPos==guiGetAreaListLen(aContext), projects[i].patches[j].selected);
              if (projectBrowserPos==guiGetAreaListLen(aContext)) ycursor=y;

//              guiSelectFillColor(aContext, 0xFFFFFF);
              if (currentProject==i && currentPatch==j) {
                TextOut(aContext->hdc, 40, y, "==>", 3);
                
              }

              drawExpandBox(aContext, 62, y, projects[i].patches[j].unfoldedPatchView);

              guiSelectPen1Color(aContext, -1);
              guiSelectFillColor(aContext, color);
              guiDrawRect(aContext, 80, y, aContext->guiClientRect.right, y+14);
              expandString(projects[i].patches[j].patchName, NULL, NULL, NULL, NULL);
              guiSelectPen1Color(aContext, textColor);
              TextOut(aContext->hdc, 80, y, expandedString, strlen(expandedString));
              projectBrowserAddHitArea(aContext, y);
              y+=14;
              if (projects[i].patches[j].unfoldedPatchView) {
                guiSelectFillColor(aContext, 0xFFFFFF);
                for(k=0;k<projects[i].patches[j].maxObjects;k++) {
                  color=projectBrowserBackground(projectBrowserPos==guiGetAreaListLen(aContext), projects[i].patches[j].objects[k].selected);
                  if (projectBrowserPos==guiGetAreaListLen(aContext)) ycursor=y;
                  guiSelectFillColor(aContext, color);
                  if (objectTitles[projects[i].patches[j].objects[k].objectType]) {
                    guiSelectPen1Color(aContext, -1);
                    guiSelectFillColor(aContext, color);
                    guiDrawRect(aContext, 100, y, aContext->guiClientRect.right, y+14);
                    expandString(objectTitles[projects[i].patches[j].objects[k].objectType], NULL, NULL, NULL, NULL);
                    guiSelectPen1Color(aContext, textColor);
                    TextOut(aContext->hdc, 100, y, expandedString, strlen(expandedString));
//                    projectBrowserAddHitArea(aContext, y);
                    y+=14;
                  }
                }
              }
            }
          }
          if (projects[i].maxTracks>0) {
            guiSelectFillColor(aContext, 15);
            TextOut(aContext->hdc, 40, y, "Tracks", 6);
            y+=14;
            for(j=0;j<projects[i].maxTracks;j++) {
              if (projects[i].tracks[j].selected) {
                color=0x00FFFF;
              } else {
                color=0xFFFFFF;
              }
              if (projectBrowserPos == guiGetAreaListLen(aContext)) {
                color-=8;
              }
              guiSelectFillColor(aContext, color);
              expandString(projects[i].tracks[j].trackName, NULL, NULL, NULL, NULL);
              TextOut(aContext->hdc, 60, y, expandedString, strlen(expandedString));
              projectBrowserAddHitArea(aContext, y);
              y+=14;
            }
          }

          if (projects[i].maxSamples>0) {
            guiSelectFillColor(aContext, 0xFFFFFF);
            TextOut(aContext->hdc, 40, y, "Samples", 7);
            y+=14;
          }
        }
      }


      if (y+guiGetVScrollPos(aContext->currentWindow)!=maxy || aContext->guiClientRect.bottom!=maxysize) {
        maxy=y+guiGetVScrollPos(aContext->currentWindow);
        maxysize=aContext->guiClientRect.bottom;
        guiVScrollRange(aContext->currentWindow, 0, maxy, maxysize);
        recalcCursor=true;
      } if (recalcCursor) {
        if (ycursor<0) {
	        guiSetVScrollPos(aContext->currentWindow, guiGetVScrollPos(aContext->currentWindow)+ycursor);
        } else if (ycursor+14>aContext->guiClientRect.bottom && aContext->guiClientRect.bottom>14) {
  	      guiSetVScrollPos(aContext->currentWindow, guiGetVScrollPos(aContext->currentWindow)+ycursor-aContext->guiClientRect.top);
        }
        recalcCursor=false;
      }
    } break;
  case GUIEVENT_MOUSEBUTTON1DOWN: {
      int i,j;
      int areaCount=0;

      for(i=0;i<maxProjects;i++) {
        areaCount++;
        if (aContext->id==areaCount) {
          if (aContext->mouseClientX<20) {
            projects[i].unfoldedProjectView = !projects[i].unfoldedProjectView;
          }
          selectProjectBrowserItem(i, NULL, -1, -1, ctrlPressed());
          transportWindowRefresh=true;
          break;
        }
        if (projects[i].unfoldedProjectView) {
          for(j=0;j<projects[i].maxTracks;j++) {
            areaCount++;
            if (aContext->id==areaCount) {
              selectProjectBrowserItem(i, NULL, -1, j, ctrlPressed());
              break;
            }
          }
          for(j=0;j<projects[i].maxPatches;j++) {
            areaCount++;
            if (aContext->id == areaCount) {
              if (aContext->mouseClientX<80) {
                projects[i].patches[j].unfoldedPatchView = !projects[i].patches[j].unfoldedPatchView;
              }
              selectProjectBrowserItem(i, &(projects[i].patches[j]), -1, -1, ctrlPressed());
              break;
            }
          }
        }
      }
    } break;
  case GUIEVENT_MOUSEBUTTON1DCLK: {
      int i,j;
      int areaCount=0;

      for(i=0;i<maxProjects;i++) {
        areaCount++;
        if (aContext->id == areaCount) {
          projects[i].unfoldedProjectView = !projects[i].unfoldedProjectView;
          projectBrowserWindowRefresh=true;
          break;
        }
        if (projects[i].unfoldedProjectView) {
          for(j=0;j<projects[i].maxTracks;j++) {
            areaCount++;
            if (aContext->id==areaCount) {
              selectProjectBrowserItem(i, NULL, -1, j, ctrlPressed());
              break;
            }
          }
          for(j=0;j<projects[i].maxPatches;j++) {
            areaCount++;
            if (aContext->id == areaCount) {
              selectProjectBrowserItem(i, &(projects[i].patches[j]), -1, -1, ctrlPressed());
              break;
            }
          }
        }
      }
    } break;
  case GUIEVENT_CHAR:
    switch(aContext->id) {
    case 3:
      // Ignore repeated events when key is held down
      if (!aContext->keyRepeatFlag) {
        saveStfToClipboard();
      }
      break;
    case 4:
      // Ignore repeated events when key is held down
      if (!aContext->keyRepeatFlag) {
        clone();
      }
      break;
    case 22:
      // Ignore repeated events when key is held down
      if (!aContext->keyRepeatFlag) {
        loadFromClipboard();
      }
      break;
    case 24:
      // Ignore repeated events when key is held down
      if (!aContext->keyRepeatFlag) {
        saveStfToClipboard();
        deleteSelected();
      }
      break;
    default:
      break;
    }
    break;
  case GUIEVENT_KEYDOWN:
    switch(aContext->id) {
    case VK_ESCAPE:
      guiHideWindow(aContext->currentWindow);
      break;
    case VK_RETURN: {
        int i,j;
        int pos=0;

        for(i=0;i<maxProjects;i++) {
          if (projectBrowserPos == pos) {
            projects[i].unfoldedProjectView = !projects[i].unfoldedProjectView;
            selectProjectBrowserItem(i, NULL, -1, -1, ctrlPressed());
            transportWindowRefresh=true;
            goto leaveProjectBrowserReturn;
          }
          pos++;
          if (projects[i].unfoldedProjectView) {
            for(j=0;j<projects[i].maxTracks;j++) {
              if (projectBrowserPos==pos) {
                selectProjectBrowserItem(i, NULL, -1, j, ctrlPressed());
                goto leaveProjectBrowserReturn;
              }
              pos++;
            }
            for(j=0;j<projects[i].maxPatches;j++) {
              if (projectBrowserPos==pos) {
                projects[i].patches[j].unfoldedPatchView = !projects[i].patches[j].unfoldedPatchView;
                selectProjectBrowserItem(i, &(projects[i].patches[j]), -1, -1, ctrlPressed());
                goto leaveProjectBrowserReturn;
              }
              pos++;
            }
          }
        }
      } leaveProjectBrowserReturn: break;
    case VK_SPACE: {
        int i,j;
        int pos=0;

        for(i=0;i<maxProjects;i++) {
          if (projectBrowserPos == pos) {
            selectProjectBrowserItem(i, NULL, -1, -1, ctrlPressed());
            transportWindowRefresh=true;
            goto leaveProjectBrowserSpace;
          }
          pos++;
          if (projects[i].unfoldedProjectView) {
            for(j=0;j<projects[i].maxTracks;j++) {
              if (projectBrowserPos==pos) {
                selectProjectBrowserItem(i, NULL, -1, j, ctrlPressed());
                goto leaveProjectBrowserSpace;
              }
              pos++;
            }
            for(j=0;j<projects[i].maxPatches;j++) {
              if (projectBrowserPos==pos) {
                selectProjectBrowserItem(i, &(projects[i].patches[j]), -1, -1, ctrlPressed());
                goto leaveProjectBrowserSpace;
              }
              pos++;
            }
          }
        }  
      } leaveProjectBrowserSpace: break;
    case VK_UP:
      if (projectBrowserPos>0) {
        projectBrowserPos--;
        projectBrowserWindowRefresh=true;
        recalcCursor=true;
      }
      break;
    case VK_DOWN:
      if (projectBrowserPos<guiGetAreaListLen(aContext)-1) {
        projectBrowserPos++;
        projectBrowserWindowRefresh=true;
        recalcCursor=true;
      }
      break;
    case VK_DELETE:
      deleteSelected();
      break;
    }
    break;
  case GUIEVENT_CLOSE:
    guiHideWindow(aContext->currentWindow);
    break;
  default:
    break;
  }
}
EOF

genWindow('projectBrowserWindow', 'PROJECTWINDOW', '"Project Browser"', 'projectBrowserHandler', undef, undef, 1, 0, undef);
